########################################################################################################
# edex_web_api_request.py
# A python api request example using the eDEx API
#
# Description:
# This script queries the API for specific data products filtered by a set of constraints.
# The data product and constraint variables are exchangeable and appendable in the PARAMS dictionary.
# A full list of data products can be found at the end of the file.
# 
# In this example, the data product filenames are returned from the /api_fetch_data GET endpoint and
# then are used in the /api_prepare_report POST endpoint.
# All data products and constraint option examples can be found at the end of the file.
#
# Author: Troy Kachor
# troyk@phys.ucalgary.ca
########################################################################################################

import requests
import time

start_time = time.time()


########################################## Request File list ###########################################
API_ENDPOINT = "https://api.edex.phys.ucalgary.ca/api_fetch_data"


# Place holder to be used in the next call to /api_prepare_report
filename_list = []

# Dictionary to pass as JSON object to the API, PARAMS must contain keys "product", "email", "result_filenames".
PARAMS = {
            'product': ['gap-los-tec-ql'], # products desired
            'date': [['2020-01-01', '2020-01-01']], # date constraint
            'gap': ['has-data'], # additional constraint
            'email': "troyk@phys.ucalgary.ca", # where data will be sent
            'result_filenames': filename_list, # placeholder for api_prepare_report below
            #### Other possible constraints. All are listed at the end of the file. All options available on edex are available here. ####
            'query-logic': ' 1 '  # applies to all PARAMS other than product, email, result_filenames. Order matters. Brackets will be followed.
          }

r = requests.get(url=API_ENDPOINT, json=PARAMS) # gets the list of files

print("--- %s seconds ---" % (time.time() - start_time)) # runtime

try: # assign the list of returned filenames to filename_list
    filename_list = r.json()["result_filenames"]
except:
    print("Could not find result_filenames key. Returned the JSON: ")
    print(r.text)
    exit()
print(filename_list) # prints the list of filenames

input("Press Enter to download files or ctrl-c to escape...") # Confirm these are the files you wish to download.


########################################### Download files via email ###########################################
API_ENDPOINT = "https://edex.phys.ucalgary.ca/api_prepare_report"

PARAMS['result_filenames'] = filename_list # update dict with returned filenames list

p = requests.post(url=API_ENDPOINT, json=PARAMS) # Send request.

# Response [200] and success message if all went well.
print(p)
print(p.text)


'''
# Dictionary example with every option available (not recommended) for picking and choosing products and constraints for the PARAMS dict

{   'result_filenames': ['CAS_Orbit_GEO_20141020T000000_20141020T235959_1.1.0.sp3.zip', <...> , 'CAS_Orbit_GEO_20141021T000000_20141021T235959_1.1.0.sp3.zip', 'RRIQL_20180912_105039_105243_4.0.1.png],
    'email': 'troyk@phys.ucalgary.ca',
    'date':  [['2020-06-01', '2020-08-01'], ['2020-11-01', '2021-01-01']]
    'product': [                        'cassiope-orbit-ephemeris-sp3',                      # products
                                        'cassiope-attitude-quaternions-and-ypr',
                                        'cassiope-bus-telemetry',
                                        'cassiope-legacy-ephemeris-and-attitude',
                                        'epop-quicklook',
                                        'epop-instrument-data-availability,
                                        'cer-quicklook',
                                        'cer-tec',
                                        'cer-ground-receiver-data',
                                        'fai-quicklook',
                                        'fai-summary',
                                        'fai-png-images',
                                        'fai-lv1-hdf5-images',
                                        'gap-quicklook',
                                        'gap-lv1',
                                        'gap-los-tec',
                                        'gap-los-tec-ql',
                                        'gap-vtec',
                                        'gap-vtec-ql',
                                        'gap-rinex-observation',
                                        'irm-quicklook',
                                        'irm-summary',
                                        'irm-surface-sensor-current',
                                        'irm-lv0b',
                                        'mgf-quicklook',
                                        'mgf-summary',
                                        'mgf-residual',
                                        'mgf-1-sps-lv1b-cdf',
                                        'mgf-160-sps-lv1b-cdf',
                                        'nms-lv0b',
                                        'nms-quicklook',
                                        'nms-quicklook-velocity',
                                        'nms-quicklook-position',
                                        'rri-quicklook',
                                        'rri-lv1-hdf5',
                                        'sei-quicklook',
                                        'sei-summary',
                                        'sei-lv0b'],
    'geophysical-parameters': [         ['ae', '0', '2000'],                            #geophysical constraints
                                        ['ap', '0', '35'],
                                        ['dst', '-600', '100'],
                                        ['f10.7', '0', '300'],
                                        ['kp10', '0', '90']],
    'imf-parameters': [                 ['absolute-b', '0', '50'],
                                        ['by-gsm', '-50', '50'],
                                        ['bz-gsm', '-50', '50'],
                                        ['flow-pressure', '0', '20'],
                                        ['flow-speed', '0', '900'],
                                        ['proton-density', '0', '100'],
                                        ['proton-temperature', '0', '1500000']],
    'e-pop-quicklook': [                'has-data'],                                    #epop constraints
    'cer': [                            'has-data',
                                        ['receiver-site', 'arecibo],
                                        ['receiver-site', 'cuzco'],
                                        ['receiver-site', 'eureka'],
                                        ['receiver-site', 'jicamara'],
                                        ['receiver-site', 'siefring'],
                                        ['receiver-site', 'pmaldonad'],
                                        ['receiver-site', 'delta-junct'],
                                        ['receiver-site', 'gakon-ak'],
                                        ['receiver-site', 'valdez-ak']],
    'fai': [                            ['camera-mode', 'high-res'],
                                        ['camera-mode', 'med-res'],
                                        ['camera-mode', 'low-res'],
                                        ['camera-source', 'infrared'],
                                        ['camera-source', 'visible'],
                                        'has-data'],
    'gap': [                            'has-data', 
                                        ['observation-rate', '100hz'],
                                        ['observation-rate', '50hz'],
                                        ['observation-rate', '20hz'],
                                        ['observation-rate', '1hz'],
                                        ['receiver-number', '0'],
                                        ['receiver-number', '1'],
                                        ['receiver-number', '2'],
                                        ['receiver-number', '3'],
                                        ['receiver-number', '4'],
                                        ['receiver-number', 'gap-a'],
                                        ['receiver-number', 'gap-o']],
    'irm': [                            'has-data'],
    'mgf': [                            'has-data'],
    'nms': [                            'has-data'],
    'rri': [                            'has-data',
                                        ['antenna', 'monopole'], 
                                        ['antenna', 'dipole'],
                                        ['bandwidth', '30khz'],
                                        ['bandwidth', '5 KHz'],
                                        ['dwell-a', '0', '60'],
                                        ['dwell-b', '0', '60'],
                                        ['frequency','0','18.5], 
                                        ['gain', 'high'],
                                        ['gain', 'medium'],
                                        ['gain', 'low'],
                                        ['output-channels', 'i1-q1-i3-q3'],
                                        ['output-channels', 'i1-i2-i3-i4'],
                                        ['sweep', 'fixed'],
                                        ['sweep', 'linear'],
                                        ['sweep', 'logarithmic']],
    'sei': [                            'has-data'],
    'spacecraft-position': [            ['altitude', '300', '1500'],
                                        ['geographic-latitude', '-81','81'],
                                        ['geographic-longitude', '-180', '180'],
                                        ['magnetic-local-time', '0', '24']],
    'spacecraft-attitude': [            ['nadir-pointing', 'true'],
                                        ['nadir-pointing', 'false'],
                                        ['pitch', '-90', '90']
                                        ['roll',  '-180', '-180'], 
                                        ['yaw',  '-180', '-180']], 
    'amateur-radio': [                  'arrl-field-day',                               #planned experiments
                                        'ham-radio']
    'auroral-region': [                 'auroral-tilt',
                                        'auroral-zone']
    'cusp-region': [                    'cusp-region'] 
    'earth-limb-viewing': [             'earth-limb'],
    'gap-orbit-determination': [        'gap-od'],
    'miscellaneous': [                  'rri-noise-survey',
                                        'rri-special-event',
                                        'rri-vlf',
                                        'churchill-vlf',
                                        'lucky-lake-vlf',
                                        'whistler-detection',
                                        'flash',
                                        'kilauea-eruption',
                                        'lucky-lake-vlf-dmsp-satellite',
                                        'rri-noise-survey-dmsp-satellite',
                                        'he+-detection',
                                        'lucky-lake-vlf-clyde-river-superdarn',
                                        'gap-uncompressed',
                                        'lucky-lake-vlf-dsx-conjunction',
                                        'rri-nw-hemisphere',
                                        'bbc-documentary',
                                        'science-campaign'],
    'point-of-interest': [              'san-francisco',
                                        'revelstoke',
                                        'portsmouth',
                                        'hurricane-dorian',
                                        'fort-mcmurray-fire',
                                        'calgary',
                                        'boston',
                                        'hurricane-sandra'],
    'polar-cap-region': [               'polar-cap'],
    'rocket-satellite-conjunctions': [  'dmsp-satellite',
                                        'cygnus-spacecraft',
                                        'cluster-satellite',
                                        'dsx-conjunction',
                                        'ici-4-rocket',
                                        'ici-5-rocket',
                                        'renu2-rocket',
                                        'techdemosat-satellite',
                                        'zacube-1-satellite'],
    'saa-region': [                     'saa-zone',
                                        'sei-saa'],
    'slew-targets': [                   'poker-flat',
                                        'fort-smith',
                                        'ice-bear-radar'],
    'solar-eclipse': [                  'solar Eclipse',
                                        'solar-eclipse-buenos-aires'],
    'swarm': [                          'swarm-a',
                                        'swarm-b',
                                        'swarm-c'],
    'terrestrial-receiver': [           'autumn-x-receiver',
                                        'lsbb-receiver',
                                        'carisma-receiver',
                                        'above-receiver']
    'terrestrial-transmitter':  [       'tiger-superdarn',
                                        'rothr',
                                        'wwv-radio-station',
                                        'wake-island-transmitter',
                                        'toolik-lake-transmitter',
                                        'dho38-radio-station',
                                        'bpm-time-signal-transmitter',
                                        'kharkiv-isr',
                                        'cyprus-transmitter',
                                        'codar',
                                        'clyde-river-superdarn',
                                        'risr-transmitter',
                                        'clyde-river-superdarn-saa-zone',
                                        'chu-radio-station',
                                        'cambridge-bay-cadi',
                                        'arecibo-transmitter',
                                        'surface-wave-radar',
                                        'sura-heater',
                                        'sondrestrom-radar',
                                        'saskatoon-superdarn',
                                        'isr-world-day',
                                        'new-mexico-transmitter',
                                        'nlk-radio-station',
                                        'nwc-radio-station',
                                        'ottawa-nrcan-transmitter',
                                        'millstone-hill-transmitter',
                                        'kodiak-superdarn',
                                        'polardarn',
                                        'prince-george-superdarn',
                                        'rankin-inlet-superdarn',
                                        'naa-transmitter',
                                        'resolute-bay-transmitter',
                                        'istok',
                                        'hawaii-transmitter',
                                        'hankasalmi-superdarn',
                                        'haarp',
                                        'eiscat-tromso',
                                        'eiscat-svalbard',
                                        'drdc']
                   }
'''
